
# LD_LIBRARY_PATH=../chiark-tcl/adns:../chiark-tcl/base valgrind --db-attach=yes tclsh t.tcl

load chiark_tcl_adns-1.so

proc pproc {x} { proc $x {args} "puts \"$x \$args\"" }
pproc pinfo
pproc yes
pproc no
pproc maybe
set adns [adns new-resolver -errcallback pinfo]
set q [eval \
	[list adns asynch yes no maybe 14 cname chiark.greenend.org.uk] \
	[list -resolver $adns]]
vwait spong
